/* -------------------------------------------------
                    mpx_8digit_demo.c

   Demo gemultiplexten 8-stelligen 7-Segmentanzeige
   mit zusaetzlich 8 gemultiplexten Tastern.

   Programm ist nur verwendbar mit ATmega mit auf
   internem Takt eingestellten Fuses, da alle
   Portpins benoetigt werden (um die Leitungen des
   UART-Anschlusses frei zu halten)

   MCU       : ATmegaxx8 - 8 MHz Takt intern

                 Fuses fuer ATmega168
                 LO-> 0xE2  / HI-> 0xdf  / EX-> 0xFF

   IDE       : keine (Editor / make)
   Toolchain : AVR-GCC

   01.02.22 by R. Seelig
  -------------------------------------------------- */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "avr_gpio.h"
#include "./mpx_8digit.h"

#define  delay            _delay_ms

#define laufspeed   100


uint8_t hallotxt[5] = { 0x76, 0x77, 0x38, 0x38, 0x3f };
uint8_t tasttxt[4]  = { 0x73, 0x3E, 0x6d, 0x76 };


/* --------------------------------------------------------
                         seg7_dimmall

     stellt fuer alle Digits denselben Helligkeitswert,
     gegeben in val, ein
   -------------------------------------------------------- */
void seg7_dimmall(uint8_t val)
{
  uint8_t i;
  
  for (i= 0; i< 8; i++) dig_pwm[i]= val;
}

/* --------------------------------------------------------
                         seg7_wiper

     laesst ein Lauflicht vorwaerts-rueckwaerts laufen
     und loescht so die Anzeige
   -------------------------------------------------------- */
void seg7_wiper(void)
{
  uint8_t i;
  
  for (i= 8; i > 0; i--)
  {
    seg7_fb[i-1]= 0x49;
    delay(laufspeed);
    seg7_fb[i-1]= 0;
  }
  for (i= 1; i< 8; i++)
  {
    seg7_fb[i]= 0x49;
    delay(laufspeed);
    seg7_fb[i]= 0;
  }
}   

/* ------------------------------------------------------------------------
                                    MAIN
   ------------------------------------------------------------------------ */
int main(void)
{
  uint32_t z;
  uint8_t i;

  mpx8_init();

  fb_clear();
  
  seg7_wiper();
  
  delay(500);
  
  seg7_dimmall(17);                   // Anzeige dunkel schalten
  
  // "Hallo" in Framebuffer schreiben
  for (i= 0; i< 5; i++)
  {
    seg7_fb[6-i]= hallotxt[i];
  }
  
  // und aufblenden
  for (i= 16; i> 0; i--) 
  {
    seg7_dimmall(i);
    delay(150);
  }   
  
  delay(1000);
  
  seg7_wiper();
  
  fb_clear();
  
  fb_setdezint(4321, 2);
  delay(1000);
  
  fb_sethexint(0x3a0f, 2);
  delay(1000);
  
  seg7_wiper();
  
  fb_clear();
  
  // "PUSH" in Framebuffer schreiben
  for (i= 0; i< 4; i++)
  {
    seg7_fb[7-i]= tasttxt[i];
  }
    
  // Tastenabfrage und anzeigen der gedrueckten Taste in
  // einer Endlosschleife
  while(1)
  {
    delay(30);
    switch (key_stat)
    {
      case key0 : seg7_fb[0]= seg7_bmp[1]; break;
      case key1 : seg7_fb[0]= seg7_bmp[2]; break;
      case key2 : seg7_fb[0]= seg7_bmp[3]; break;
      case key3 : seg7_fb[0]= seg7_bmp[4]; break;
      case key4 : seg7_fb[0]= seg7_bmp[5]; break;
      case key5 : seg7_fb[0]= seg7_bmp[6]; break;
      case key6 : seg7_fb[0]= seg7_bmp[7]; break;
      case key7 : seg7_fb[0]= seg7_bmp[8]; break;
      default   : seg7_fb[0]= seg7_bmp[0]; break;
    }    
  }

}

